(* runtime configuration *)
IMPLEMENTATION MODULE Config;

IMPORT Str, FIO, FIOStd, Lib2, Lib3, Args, StrOut;

CONST
  prg = 'ChgCR';
  ver = 3;


PROCEDURE WrInfo( VAR b: StrOut.TBuffer );
BEGIN
  StrOut.WrStr( b, prg );
  StrOut.WrStr( b, ' v' );
  StrOut.WrCard( b, ver, 0 );
  StrOut.WrStr( b, ': Change CRs in ASCII text files.' );
END WrInfo;


PROCEDURE ParseArgs();

  PROCEDURE Help();
  CONST
    f ::= FIOStd.out;
  VAR
    b: StrOut.TBuffer;
    s: ARRAY [0..127] OF CHAR;
  BEGIN
    StrOut.Init( b, s );
    StrOut.Clear( b ); StrOut.WrStr( b, "(* " ); WrInfo( b ); StrOut.WrStr( b, " *)" ); StrOut.WrLn( b );
    FIO.WrStr( f, s );
    StrOut.Clear( b ); StrOut.WrStr( b, "syntax = '" ); StrOut.WrStr( b, prg ); StrOut.WrStr( b, "' { option } { file } ." ); StrOut.WrLn( b );
    FIO.WrStr( f, s );
    StrOut.Clear( b ); StrOut.WrStr( b, "option = '-h'     (* show this info and exit *)" ); StrOut.WrLn( b );
    FIO.WrStr( f, s );
    StrOut.Clear( b ); StrOut.WrStr( b, "       | '-d'     (* show debug messages *)" ); StrOut.WrLn( b );
    FIO.WrStr( f, s );
    StrOut.Clear( b ); StrOut.WrStr( b, "       | '-t'     (* set current time *)" ); StrOut.WrLn( b );
    FIO.WrStr( f, s );
    StrOut.Clear( b ); StrOut.WrStr( b, "       | '-u'     (* UNIX destination format (default DOS) *)" ); StrOut.WrLn( b );
    FIO.WrStr( f, s );
    StrOut.Clear( b ); StrOut.WrStr( b, "       | '-v'     (* be verbose *) ." ); StrOut.WrLn( b );
    FIO.WrStr( f, s );
    Lib3.SetReturnCode( 0 );
    HALT;
  END Help;

  PROCEDURE Error( t-: ARRAY OF CHAR );
  VAR
    b: StrOut.TBuffer;
    s: ARRAY [0..127] OF CHAR;
  BEGIN
    StrOut.Init( b, s );
    StrOut.WrStr( b, prg );
    StrOut.WrStr( b, '.Config.ParseArgs.Error: ' );
    StrOut.WrStr( b, t );
    IF Args.len > 0 THEN
      StrOut.WrStr( b, ": '" );
      StrOut.WrStr( b, Args.lit );
      StrOut.WrStr( b, "'" );
    END;
    StrOut.WrLn( b );
    StrOut.WrStr( b, "Try '" );
    StrOut.WrStr( b, prg );
    StrOut.WrStr( b, " -h' for help." );
    Lib3.FatalError( s );
  END Error;

  PROCEDURE Options();
  BEGIN
    WHILE Args.sym = Args.symOpt DO
      IF    Str.Compare( Args.lit, '?' ) = 0 THEN
        Help();
      ELSIF Str.Compare( Args.lit, 'h' ) = 0 THEN
        Help();
      ELSIF Str.Compare( Args.lit, 'd' ) = 0 THEN
        data.dbg := TRUE;
      ELSIF Str.Compare( Args.lit, 't' ) = 0 THEN
        data.tim := TRUE;
      ELSIF Str.Compare( Args.lit, 'u' ) = 0 THEN
        data.unx := TRUE;
      ELSIF Str.Compare( Args.lit, 'v' ) = 0 THEN
        data.vrb := TRUE;
      ELSE
        Error( 'Invalid option' );
      END;
      Args.Read();
    END;
  END Options;

  PROCEDURE File();
  BEGIN
    IF data.cnt >= cntMax THEN Error( 'Too many file names' ) END;
    INC( data.cnt );
    IF Args.len > SIZE( data.fls[ data.cnt ] ) THEN Error( 'File name too long' ) END;
    Str.Copy( data.fls[ data.cnt ], Args.lit );
    Args.Read();
  END File;

BEGIN
  Args.Reset();
  Args.Read();
  Options();
  WHILE Args.sym = Args.symArg DO
    File();
    Options();
  END;
  IF Args.sym # Args.symEOA THEN Error( 'Unexpected argument' ) END;
END ParseArgs;

PROCEDURE InitDir();
VAR
  drv: SHORTCARD;
BEGIN
  drv := FIO.GetDrive();
  FIO.GetDir( drv, data.dir );
  Str.Prepend( data.dir, ':' );
  Str.Prepend( data.dir, CHR( ORD( 'A' ) + CARDINAL(drv-1) ) );
END InitDir;

PROCEDURE Show( screen: BOOLEAN );
CONST
  f ::= FIOStd.out;
VAR
  b: StrOut.TBuffer;
  s: ARRAY [0..127] OF CHAR;

  PROCEDURE Out();
  BEGIN
    (*
    Rep.WrStr( s ); Rep.WrLn();
    *)
    IF ~ screen THEN RETURN END;
    FIO.WrStr( f, s ); FIO.WrLn( f );
  END Out;

  PROCEDURE ShowHdr();
  BEGIN
    StrOut.Clear( b ); WrInfo( b ); Out();
    StrOut.Clear( b ); StrOut.WrStr( b, prg ); StrOut.WrStr( b, '.Config.data:' ); Out();
  END ShowHdr;

  PROCEDURE Begin( n-: ARRAY OF CHAR );
  CONST w = 3;
  BEGIN
    StrOut.Clear( b ); StrOut.WrStr( b, '  ' ); StrOut.WrStrAdj( b, n, -w ); StrOut.WrStr( b, ' : ' );
  END Begin;

  PROCEDURE ShowStr( n-: ARRAY OF CHAR; v-: ARRAY OF CHAR );
  BEGIN
    Begin( n ); StrOut.WrChar( b, "'" ); StrOut.WrStr( b, v ); StrOut.WrChar( b, "'" ); Out();
  END ShowStr;

  PROCEDURE ShowCard( n-: ARRAY OF CHAR; v: CARDINAL );
  CONST w = 1;
  BEGIN
    Begin( n ); StrOut.WrCard( b, v, w ); Out();
  END ShowCard;

VAR
  i: CARDINAL;
BEGIN
  StrOut.Init( b, s );
  ShowHdr();
  ShowStr(  'prg', data.prg );
  ShowCard( 'ver', data.ver );
  ShowCard( 'dbg', ORD( data.dbg ) );
  ShowCard( 'vrb', ORD( data.vrb ) );
  ShowCard( 'tim', ORD( data.tim ) );
  ShowCard( 'unx', ORD( data.unx ) );
  ShowStr(  'dir', data.dir );
  ShowCard( 'cnt', data.cnt );
  FOR i := 1 TO data.cnt DO
    ShowStr(  'fil', data.fls[i] );
  END;
END Show;

PROCEDURE InitModule();
BEGIN
  Lib2.Fill( data, 0 );
  Str.Copy( data.prg, prg );
  data.ver := ver;
  data.dbg := FALSE;
  ParseArgs();
  InitDir();
  Show( data.vrb );
END InitModule;

BEGIN
  InitModule();
END Config.