MODULE ChgCR;

IMPORT Lib, FIO, Str, Args;

CONST
  verStr = 'ChgCR 2014-02-07';
  kB = 1024;
  MB = 1024 * kB;

  noFile = MAX( FIO.File );
  tmpFn = '$$ChgCR.$$$';

  cLF = CHR( 0AH );
  cCR = CHR( 0DH );


TYPE
  TFileName = ARRAY [ 0 .. 255 ] OF CHAR;
  TFileBuffer = ARRAY [ 1 .. 8 * kB ] OF BYTE;

VAR
  verbose: BOOLEAN;
  add: BOOLEAN;
  keepTime: BOOLEAN;
  in, out: FIO.File;
  inFn: TFileName;
  inBuf, outBuf: TFileBuffer;
  count: LONGCARD;


PROCEDURE InitGlobals();
BEGIN
  verbose := FALSE;
  add := FALSE;
  keepTime := FALSE;
  in := noFile;
  out := noFile;
  inFn := '';
  count := 0;
END InitGlobals;


PROCEDURE Exit( x: SHORTCARD );
BEGIN
  Lib.SetReturnCode( x );
  HALT;
END Exit;

PROCEDURE Error( s0, s1: ARRAY OF CHAR );
CONST
  f = FIO.ErrorOutput;
BEGIN
  FIO.WrStr( f, 'ChgCR.Error: ' );
  FIO.WrStr( f, s0 );
  IF Str.Length( s1 ) > 0 THEN
    FIO.WrStr( f, ': "' );
    FIO.WrStr( f, s1 );
    FIO.WrStr( f, '"' );
  END;
  FIO.WrStr( f, '.' );
  FIO.WrLn( f );
  FIO.WrStr( f, 'Try -h for help.' );
  FIO.WrLn( f );
  Exit( 1 );
END Error;

PROCEDURE Info( s0, s1: ARRAY OF CHAR );
CONST
  f = FIO.ErrorOutput;
BEGIN
  IF verbose THEN FIO.WrLn( f ) END;
  FIO.WrStr( f, 'ChgCR.Info: ' );
  FIO.WrStr( f, s0 );
  IF Str.Length( s1 ) > 0 THEN
    FIO.WrStr( f, ': "' );
    FIO.WrStr( f, s1 );
    FIO.WrStr( f, '"' );
  END;
  FIO.WrStr( f, '.' );
  FIO.WrLn( f );
END Info;

PROCEDURE InfoCard( s: ARRAY OF CHAR; v: LONGCARD );
CONST
  f = FIO.ErrorOutput;
BEGIN
  IF verbose THEN FIO.WrLn( f ) END;
  FIO.WrStr( f, 'ChgCR.Info: ' );
  FIO.WrStr( f, s );
  FIO.WrStr( f, ' ' );
  FIO.WrLngHex( f, v, 0 );
  FIO.WrStr( f, 'H.' );
  FIO.WrLn( f );
END InfoCard;

PROCEDURE Version();
CONST
  f = FIO.ErrorOutput;
BEGIN
  FIO.WrStr( f, verStr );
  FIO.WrLn( f );
END Version;

PROCEDURE Help();
CONST
  f = FIO.ErrorOutput;
BEGIN
  FIO.WrStr( f, verStr );
  FIO.WrStr( f, ': Add or remove CR.' );
  FIO.WrLn( f );
  FIO.WrStr( f, 'syntax = ChgCR mode {option} file .' );
  FIO.WrLn( f );
  FIO.WrStr( f, 'mode = a | r . (* add or remove *)' );
  FIO.WrLn( f );
  FIO.WrStr( f, 'option = "-V"  (* show version and exit *)' );
  FIO.WrLn( f );
  FIO.WrStr( f, '       | "-h"  (* show this text and exit *)' );
  FIO.WrLn( f );
  FIO.WrStr( f, '       | "-v"  (* be verbose *)' );
  FIO.WrLn( f );
  FIO.WrStr( f, '       | "-t"  (* keep timestamp *)' );
  FIO.WrStr( f, ' .' );
  FIO.WrLn( f );
  Exit( 0 );
END Help;

PROCEDURE Summary();
CONST
  f = FIO.ErrorOutput;
BEGIN
  FIO.WrStr( f, 'ChgCR: "' );
  FIO.WrStr( f, inFn );
  FIO.WrStr( f, '": ' );
  FIO.WrLngCard( f, count , 0 );
  FIO.WrStr( f, ' CRs ' );
  IF add THEN
    FIO.WrStr( f, 'added.' );
  ELSE
    FIO.WrStr( f, 'removed.' );
  END;
  FIO.WrLn( f );
END Summary;


PROCEDURE Init();

  PROCEDURE Options();
  BEGIN
    WHILE Args.sym = Args.symOpt DO
      IF Str.Compare( Args.lit, 'V' ) = 0 THEN
        Version(); Exit( 0 );
      ELSIF Str.Compare( Args.lit, '?' ) = 0 THEN
        Help();
      ELSIF Str.Compare( Args.lit, 'h' ) = 0 THEN
        Help();
      ELSIF Str.Compare( Args.lit, 'v' ) = 0 THEN
        verbose := TRUE;
      ELSIF Str.Compare( Args.lit, 't' ) = 0 THEN
        keepTime := TRUE;
      END;
      Args.Read();
    END;
  END Options;

  PROCEDURE ParseArgs();
  BEGIN
    Args.Reset();
    Args.Read();
    Options();

    IF Args.sym # Args.symArg THEN Error( 'Expected mode', '' ) END;
    IF Str.Compare( Args.lit, 'a' ) = 0 THEN
      add := TRUE;
    ELSIF Str.Compare( Args.lit, 'r' ) = 0 THEN
      add := FALSE;
    ELSE
      Error( 'Expected mode', '' );
    END;
    Args.Read();
    Options();

    IF Args.sym # Args.symArg THEN Error( 'Expected file name', '' ) END;
    IF ~ FIO.Exists( Args.lit ) THEN Error( 'File does not exists', Args.lit ) END;
    Str.Copy( inFn, Args.lit );
    Args.Read();
    Options();
  END ParseArgs;

BEGIN
  Lib.EnableBreakCheck();
  Lib.SetReturnCode( 1 );
  InitGlobals();
  ParseArgs();
END Init;


PROCEDURE Main();

  PROCEDURE Open();
  BEGIN
    in := FIO.OpenRead( inFn );
    IF in = noFile THEN Error( 'Cannot open', inFn ) END;
    FIO.AssignBuffer( in, inBuf );
    out := FIO.Create( tmpFn );
    IF out = noFile THEN Error( 'Cannot create', tmpFn ) END;
    FIO.AssignBuffer( out, outBuf );
  END Open;

  INLINE PROCEDURE Write( c: CHAR );
  BEGIN
    FIO.WrBin( out, c, 1 );
  END Write;

  PROCEDURE Add();
  VAR
    c: CHAR;
    n: CARDINAL;
  BEGIN
    n := FIO.RdBin( in, c, 1 );
    WHILE n > 0 DO
      IF c = cLF THEN Write( cCR ); INC( count ) END;
      Write( c );
      n := FIO.RdBin( in, c, 1 );
    END;
  END Add;

  PROCEDURE Remove();
  VAR
    c: CHAR;
    n: CARDINAL;
  BEGIN
    n := FIO.RdBin( in, c, 1 );
    WHILE n > 0 DO
      IF c = cCR THEN INC( count ) ELSE Write( c ) END;
      n := FIO.RdBin( in, c, 1 );
    END;
  END Remove;

  PROCEDURE CopyTime();
  VAR d: LONGCARD;
  BEGIN
    d := FIO.GetFileDate( in );
    FIO.SetFileDate( out, d );
  END CopyTime;

  PROCEDURE Close();
  BEGIN
    FIO.Close( out );
    FIO.Close( in );
  END Close;

  PROCEDURE Rename();
  BEGIN
    FIO.Erase( inFn );
    FIO.Rename( tmpFn, inFn );
  END Rename;

BEGIN
  Open();
  IF add THEN Add() ELSE Remove() END;
  IF keepTime THEN CopyTime() END;
  Close();
  Rename();
END Main;


PROCEDURE Done();
BEGIN
  Summary();
  Lib.SetReturnCode( 0 );
END Done;


BEGIN
  Init();
  Main();
  Done();
END ChgCR.
